'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.regexes = exports.nul = exports.undef = exports.object = exports.array = exports.boolean = exports.func = exports.number = exports.string = exports.oneOfType = exports.format = exports.oneOf = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _parseFunction = require('./support/parseFunction');

var _parseFunction2 = _interopRequireDefault(_parseFunction);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var type = function type(comp) {
  if (comp instanceof Array) {
    return 'array';
  } else if (comp === null) {
    return 'null';
  }
  return typeof comp === 'undefined' ? 'undefined' : _typeof(comp);
};

var functionName = function functionName(f) {
  return (0, _parseFunction2.default)(f).name.replace('_', '').replace('Internal', '');
};

var oneOf = exports.oneOf = function oneOf(array) {
  return function oneOfInternal(comp) {
    return array.includes(comp);
  };
};
var format = exports.format = function format(regex) {
  return function formatInternal(comp) {
    return regex.test(comp);
  };
};
var oneOfType = exports.oneOfType = function oneOfType(types) {
  return function oneOfTypeInternal(comp) {
    return types.filter(function (typeFunc) {
      return typeFunc(comp);
    }).length !== 0;
  };
};

var _string = function _string(comp) {
  return type(comp) === 'string';
};
exports.string = _string;
var _number = function _number(comp) {
  return type(comp) === 'number';
};
exports.number = _number;
var _func = function _func(comp) {
  return type(comp) === 'function';
};
exports.func = _func;
var _boolean = function _boolean(comp) {
  return type(comp) === 'boolean';
};
exports.boolean = _boolean;
var _array = function _array(comp) {
  return type(comp) === 'array';
};
exports.array = _array;
var _object = function _object(comp) {
  return type(comp) === 'object';
};
exports.object = _object;
var _undef = function _undef(comp) {
  return type(comp) === 'undefined';
};
exports.undef = _undef;
var _nul = function _nul(comp) {
  return type(comp) === 'null';
};

exports.nul = _nul;

var Shape = function () {
  function Shape(shape) {
    _classCallCheck(this, Shape);

    this.shape = shape;
    this.printableShapeObject = type(shape) === 'array' ? [] : {};
    this.printableShape = this.calculatePrintableShape(shape);
    this.nonMatchingMessages = [];
    this.previousNonMatchingMessages = [];
  }

  _createClass(Shape, [{
    key: 'lastNonMatches',
    value: function lastNonMatches() {
      return this.previousNonMatchingMessages[this.previousNonMatchingMessages.length - 1];
    }
  }, {
    key: 'matches',
    value: function matches(object) {
      this.recursivelyAccumulateNonMatches(object, this.shape);
      if (this.nonMatchingMessages.length) {
        // Store and reset non-matching messages, for future assertions
        this.previousNonMatchingMessages.push(this.nonMatchingMessages);
        this.nonMatchingMessages = [];
        return false;
      }
      return true;
    }

    /////////////////
    //   PRIVATE   //
    /////////////////

  }, {
    key: 'recursivelyAccumulateNonMatches',
    value: function recursivelyAccumulateNonMatches(object, shape) {
      if (shape instanceof Array) {
        this.testArrayMatch(shape, object);
      } else if (type(shape) === 'object') {
        this.testObjectMatch(shape, object);
      } else if (type(shape) === 'function') {
        var funcName = functionName(shape);
        var logAndTestFunction = this[funcName](shape);
        logAndTestFunction(object);
      }

      return true;
    }
  }, {
    key: 'testObjectMatch',
    value: function testObjectMatch(shape, object) {
      var _this = this;

      if ((typeof object === 'undefined' ? 'undefined' : _typeof(object)) === 'object') {
        _lodash2.default.keys(shape).forEach(function (k) {
          return _this.recursivelyAccumulateNonMatches(object[k], shape[k]);
        });
      } else {
        this.nonMatchingMessages.push(JSON.stringify(object) + ' is ' + this.articulate(typeof object === 'undefined' ? 'undefined' : _typeof(object)) + ' ' + (typeof object === 'undefined' ? 'undefined' : _typeof(object)) + ', not an object');
      }
    }
  }, {
    key: 'testArrayMatch',
    value: function testArrayMatch(shape, object) {
      var _this2 = this;

      if (object instanceof Array) {
        object.forEach(function (child) {
          return _this2.recursivelyAccumulateNonMatches(child, shape[0]);
        });
      } else {
        this.nonMatchingMessages.push(JSON.stringify(object) + ' is ' + this.articulate(typeof object === 'undefined' ? 'undefined' : _typeof(object)) + ' ' + (typeof object === 'undefined' ? 'undefined' : _typeof(object)) + ', not an array');
      }
    }
  }, {
    key: 'oneOf',
    value: function oneOf(testFunc) {
      var _this3 = this;

      return function (comp) {
        _this3.logFailure(testFunc, comp, '"' + comp + '" is not within the specified array');
      };
    }
  }, {
    key: 'oneOfType',
    value: function oneOfType(testFunc) {
      var _this4 = this;

      return function (comp) {
        _this4.logFailure(testFunc, comp, JSON.stringify(comp) + ' is ' + _this4.articulate(type(comp)) + ' ' + type(comp) + ', which is not among the specified types');
      };
    }
  }, {
    key: 'format',
    value: function format(testFunc) {
      var _this5 = this;

      return function (comp) {
        _this5.logFailure(testFunc, comp, '"' + comp + '" does not match given regex');
      };
    }
  }, {
    key: 'string',
    value: function string() {
      var _this6 = this;

      return function (comp) {
        _this6.logFailure(_string, comp, _this6.errorString(comp, 'string'));
      };
    }
  }, {
    key: 'number',
    value: function number() {
      var _this7 = this;

      return function (comp) {
        _this7.logFailure(_number, comp, _this7.errorString(comp, 'number'));
      };
    }
  }, {
    key: 'func',
    value: function func() {
      var _this8 = this;

      return function (comp) {
        _this8.logFailure(_func, comp, _this8.errorString(comp, 'function'));
      };
    }
  }, {
    key: 'boolean',
    value: function boolean() {
      var _this9 = this;

      return function (comp) {
        _this9.logFailure(_boolean, comp, _this9.errorString(comp, 'boolean'));
      };
    }
  }, {
    key: 'undef',
    value: function undef() {
      var _this10 = this;

      return function (comp) {
        _this10.logFailure(_undef, comp, _this10.errorString(comp, 'undefined'));
      };
    }
  }, {
    key: 'nul',
    value: function nul() {
      var _this11 = this;

      return function (comp) {
        _this11.logFailure(_nul, comp, _this11.errorString(comp, 'null'));
      };
    }
  }, {
    key: 'array',
    value: function array() {
      var _this12 = this;

      return function (comp) {
        _this12.logFailure(_array, comp, _this12.errorString(comp, 'array'));
      };
    }
  }, {
    key: 'object',
    value: function object() {
      var _this13 = this;

      return function (comp) {
        _this13.logFailure(_object, comp, _this13.errorString(comp, 'object', type(comp)));
      };
    }
  }, {
    key: 'logFailure',
    value: function logFailure(testFunc, comp, errorMsg) {
      if (!testFunc(comp)) {
        this.nonMatchingMessages.push(errorMsg);
      }
    }
  }, {
    key: 'articulate',
    value: function articulate(string) {
      var vowels = ['a', 'e', 'i', 'o', 'u'];
      return vowels.includes(string[0]) ? 'an' : 'a';
    }
  }, {
    key: 'errorString',
    value: function errorString(comp, target) {
      var givenType = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

      var type = givenType || (typeof comp === 'undefined' ? 'undefined' : _typeof(comp));

      return JSON.stringify(comp) + ' is ' + this.articulate(type) + ' ' + type + ', not ' + this.articulate(target) + ' ' + target;
    }
  }, {
    key: 'calculatePrintableShape',
    value: function calculatePrintableShape(shape) {
      var startingObject = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

      var replaceValues = function replaceValues(k, value) {
        return typeof value === 'function' ? functionName(value) : value;
      };
      var objectString = JSON.stringify(shape, replaceValues).replace(/"nul"/g, '"null"').replace(/"undef"/g, '"undefined"').replace(/"func"/g, '"function"');

      return JSON.parse(objectString);
    }
  }]);

  return Shape;
}();

var regexes = exports.regexes = {
  iso8601: /\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}[A-Z]{1}/
};

exports.default = Shape;